# coding: utf8
import random
import copy

# ------------------------
# OneMax問題をGAで解いてみる
# ------------------------

# ------------------------
# パラメータ
LIST_SIZE = 10       # 0/1のリスト長（遺伝子長）

POPULATION_SIZE = 10 # 集団の個体数
GENERATION = 25      # 世代数
MUTATE = 0.1         # 突然変異の確率
SELECT_RATE = 0.5    # 選択割合

# ------------------------
# 適応度を計算する
def calc_fitness(individual):
  return sum(individual)  # リスト要素の合計

# ------------------------
# 集団を適応度順にソートする
def sort_fitness(population):
  # 適応度と個体をタプルにしてリストへ格納する
  fp = []
  for individual in population:
    fitness = calc_fitness(individual)
    fp.append((fitness, individual))
  fp.sort(reverse=True)  # 適用度でソート(降順)

  # ソートした個体を格納する集団
  sorted_population = []  
  
  # 個体を取り出し、集団に格納する
  for fitness, individual in fp:
    sorted_population.append(individual)
  return sorted_population

# ------------------------
# （2）選択
# 適応度の高い個体を残す
def selection(population):
  sorted_population = sort_fitness(population)
  # 残す個体数。このプログラム
  # のパラメータでは、nは常に5
  n = int(POPULATION_SIZE * SELECT_RATE)  
  return sorted_population[0:n]

# ------------------------
# （3）交叉
# まず、ind1をコピーして新しい個体indを作る
# その後、ランダムに決めたr1～r2の範囲を
# ind2の遺伝子に置き換える
def crossover(ind1, ind2):
  r1 = random.randint(0, LIST_SIZE - 1)
  r2 = random.randint(r1 + 1, LIST_SIZE)
  ind = copy.deepcopy(ind1)
  ind[r1:r2] = ind2[r1:r2]
  return ind

# ------------------------
# （4）突然変異
# 突然変異確率(MUTATE)に従って突然変異させる
def mutation(ind1):
  ind2 = copy.deepcopy(ind1)
  for i in range(LIST_SIZE):
    if random.random() < MUTATE:
      ind2[i] = random.randint(0, 1)
  return ind2

# ------------------------
# メイン処理

# （1）初期集団を生成
population = []  # 集団
for i in range(POPULATION_SIZE):
  individual = []    # 個体
  for j in range(LIST_SIZE):
    # 0/1を乱数で追加
    individual.append(random.randint(0, 1))  
  population.append(individual)

for generation in range(GENERATION):
  print(str(generation + 1) + u"世代")
  
  # （2）選択
  population = selection(population)
  
  # 交叉と突然変異を繰り返す
  # 交叉と突然変異で増やす個体数
  # このプログラムのパラメータでは、nは常に5
  n = POPULATION_SIZE - len(population)  
  for i in range(n):
    # 集団から2個体をランダムに選び、
    # 交叉した個体を生成する
    r1 = random.randint(0, len(population) - 1)
    r2 = random.randint(0, len(population) - 1)
    # （3）交叉
    individual = \
      crossover(population[r1], population[r2])

    # （4）突然変異
    individual = mutation(individual)  

    # 集団に追加する
    population.append(individual)  
  
  for individual in population:
    print(individual)
  


