import os, glob, sys
import numpy as np
from keras.preprocessing import image
import time
import glob

MAKE_IMAGES = 10 # 何倍に水増しするかの設定（倍率）
INTERVAL = 0.1 # 画像生成処理のインターバル

def augmentImage(x):
  # 画像の水増しに使うフィルターを定義
  datagenX = image.ImageDataGenerator(
    # -20度～20度の範囲でランダムに回転を行う
    rotation_range = 20,
    # 上下平行移動
    height_shift_range = 0.3,
    # 左右平行移動
    width_shift_range = 0.3,
    # -10度～10度の範囲でランダムにせん断
    shear_range = 10,
    # [0.5, 1.2]の範囲でランダムに拡大縮小する
    zoom_range = [0.5, 1.2],
    # [-25.0, 25.0]の範囲でランダムに画素値に値を足す
    channel_shift_range = 25.,
    # 水平方向へのランダムの反転
    horizontal_flip = False,
    # 垂直方向へのランダムの反転
    vertical_flip = False
  )

  # 画像の水増しを行うジェネレータを準備
  genX = datagenX.flow(x, batch_size=1, save_to_dir=loadAndSavePath,
             save_prefix='aData', save_format='jpg')

  return genX

# 起動時の引数で、画像生成のキーワードを指定する
# 画像生成のキーワード＝フォルダー名

if len(sys.argv) < 2:
  print('キーワードが未指定です。')
else:
  # 引数からキーワードを取得する
  keyword = sys.argv[1]
  loadAndSavePath = "./" + keyword

  # 元画像の読み込み
  imageFiles = glob.glob(os.path.join(loadAndSavePath, '*.jpg'))
  # 元画像のファイル数
  motoFileCnt = len(imageFiles)

  print('元の画像数={}'.format(motoFileCnt))

  cnt = 0
  for file in imageFiles:
    print('({}/{})生成元ファイル={}'.format(cnt+1, motoFileCnt, file))
    cnt += 1
    img = image.load_img(file)
    img = np.array(img)
    genX = augmentImage(img[np.newaxis])
    # 画像生成の実行
    breakCount = cnt * MAKE_IMAGES + motoFileCnt
    for i in range(MAKE_IMAGES):
      next(genX)
      time.sleep(INTERVAL)

    # next関数の処理が終わる前に上記for文を抜けた場合は
    # 下記while文で不足分を補う
    while (len(glob.glob(loadAndSavePath+'/*.jpg')) < breakCount):
      time.sleep(INTERVAL)
      next(genX)